#!/usr/bin/python3

import argparse
import json
import pathlib
import subprocess
import tempfile

# spanish order
SPA_ORDER = [
    'Spanish',
    'European Spanish',
    'European',
]


def _sort_spa(item):
    """Return a number to use to sort the SPA subs."""
    name = item['properties'].get('track_name')
    try:
        return SPA_ORDER.index(name)
    except ValueError:
        return 500


def _get_mkv_tracks(moviefilepath, audio_track):
    """Get the tracks from the mkv."""
    cmd = ["mkvmerge", "-F", "json", "-i", str(moviefilepath)]
    cp = subprocess.run(cmd, stdout=subprocess.PIPE, encoding='utf8')
    cp.check_returncode()
    data = json.loads(cp.stdout)

    video = None
    audios = []
    subs = []
    for track in data['tracks']:
        if track['type'] == 'video':
            if video is not None:
                print("ERROR: multiple video tracks")
                exit()
            video = track['id']
        elif track['type'] == 'audio':
            audios.append(track['id'])
        elif track['type'] == 'subtitles' and track['properties']['language'] == 'spa':
            subs.append(track)

    if video is None or len(audios) == 0 or len(subs) == 0:
        print("ERROR: missing something: video={} audios={} subs={}".format(video, audios, subs))
        exit()

    if len(subs) == 1:
        track = subs[0]
    else:
        ordered_subs = sorted(subs, key=_sort_spa)
        track = ordered_subs[0]
    sub = track['id']

    if len(audios) == 1:
        audio = audios[0]
    elif audio_track is not None:
        if audio_track not in audios:
            print(
                "ERROR: indicated audio {!r} not in found tracks: {}".format(audio_track, audios))
            exit()
        audio = audio_track
    else:
        print("ERROR: multiple audio tracks: {}".format(audios))
        exit()

    return video, audio, sub


def _extract_subtitle(moviefilepath, audio_track):
    """Get the cleaned subtitle from the file."""
    cmd = [
        "ffprobe", "-v", "quiet",
        "-print_format", "json", "-show_format", "-show_streams",
        str(moviefilepath),
    ]
    cp = subprocess.run(cmd, stdout=subprocess.PIPE, encoding='utf8')
    cp.check_returncode()
    data = json.loads(cp.stdout)

    subs = None
    for stream in data['streams']:
        if stream['codec_type'] == 'subtitle' and stream['tags']['language'] == 'spa':
            if subs is None:
                # get the first one
                subs = stream

    if not subs:
        print("ERROR: missing subs")
        exit()

    # no way to differenciate which sub is better if we have several ones, just get the first one
    # and export it to a file
    tmp = tempfile.NamedTemporaryFile(suffix=".srt")
    tmp.close()
    sub_filepath = tmp.name
    cmd = [
        "ffmpeg",
        "-loglevel", "warning",
        "-i", moviefilepath,
        "-map", "0:{}".format(subs['index']),
        "-scodec", "srt",
        sub_filepath,
    ]
    subprocess.run(cmd)
    cmd = ["subtitles.py", "check", sub_filepath]
    subprocess.run(cmd)

    return sub_filepath


def _process_mkv(moviefile, outpath, audio_track):
    """Process a mkv file."""
    video_track, audio_track, sub_track = _get_mkv_tracks(moviefile, audio_track)

    # smash together only the video, audio and sub tracks
    cmd = [
        "mkvmerge",
        "--no-buttons", "--no-track-tags", "--no-chapters",
        "--video-tracks", str(video_track),
        "--audio-tracks", str(audio_track),
        "--subtitle-tracks", str(sub_track),
        "--default-track", str(sub_track),
        "--forced-track", str(sub_track),
        str(moviefile),
        "--output", outpath,
    ]
    subprocess.run(cmd)


def _process_mp4(moviefile, outpath):
    """Process a mp4 file."""
    sub_filepath = _extract_subtitle(moviefile)
    cmd = ["merge_video_subs.py", "--name", outpath, moviefile, sub_filepath]
    subprocess.run(cmd)


def main(moviefiles, audio_track):
    """Main entry point."""
    for mf in moviefiles:
        print("--- Procesing", mf)
        extension = mf.suffix.lower()
        if extension == '.mp4':
            process_func = _process_mp4
        elif extension == '.mkv':
            process_func = _process_mkv
        else:
            print("ERROR: unsupported extension {!r} in file {}".format(extension, mf))
            continue

        # in both cases the result ends being a Matroska file
        outpath = mf.parent / (mf.stem + '--normalized.mkv')
        process_func(mf, outpath, audio_track)


USAGE = """
  Usage: normalize_video_subs.py file1.mkv [file2.mkv, [...]]
"""

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--audio-track", type=int, help="The audio track to choose.")
    parser.add_argument("filepath", type=pathlib.Path, nargs="+", help="The files to process.")
    args = parser.parse_args()

    main(args.filepath, args.audio_track)
